/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>

/**
 * Calculate the distance between two points in 2D space.
 * 
 * @param x0 First Position X Coordinate.
 * @param y0 First Position Y Coordinate.
 * @param x1 Second Position X Coordinate.
 * @param y1 Second Position Y Coordinate.
 * @return The distance measurement.
 */
float vectorDistance2D(float x0, float y0, float x1, float y1);

/**
 * Calculate the distance between two points in 3D space.
 * 
 * @param x0 First Position X Coordinate.
 * @param y0 First Position Y Coordinate.
 * @param z0 First Position Z Coordinate.
 * @param x1 Second Position X Coordinate.
 * @param y1 Second Position Y Coordinate.
 * @param z1 Second Position Z Coordinate.
 * @return The distance measurement.
 */
float vectorDistance3D(
  float x0, float y0, float z0, float x1, float y1, float z1
);